import { Observable } from 'rxjs';
import { NgxUiLoaderConfig } from './ngx-ui-loader-config';
export declare class NgxUiLoaderService {
    private config;
    private _defaultConfig;
    private _waitingForeground;
    private _waitingBackground;
    private _showForeground;
    private _showBackground;
    private _foregroundClosing;
    private _backgroundClosing;
    /**
     * For internal use
     * @docs-private
     */
    showForeground: Observable<boolean>;
    /**
     * For internal use
     * @docs-private
     */
    showBackground: Observable<boolean>;
    /**
     * For internal use
     * @docs-private
     */
    foregroundClosing: Observable<boolean>;
    /**
     * For internal use
     * @docs-private
     */
    backgroundClosing: Observable<boolean>;
    private _onStart;
    private _onStop;
    private _onStopAll;
    /**
     * Emit when a loading is started
     */
    onStart: Observable<any>;
    /**
     * Emit when a loading is stopped
     */
    onStop: Observable<any>;
    /**
     * Emit when all loadings are stopped
     */
    onStopAll: Observable<any>;
    /**
     * Constructor
     * @param config
     */
    constructor(config: NgxUiLoaderConfig);
    /**
     * Get default loader configuration
     * @returns default configuration object
     */
    getDefaultConfig(): NgxUiLoaderConfig;
    /**
     * Get current status
     * @returns An object with waiting foreground and background properties
     */
    getStatus(): {
        waitingForeground: {};
        waitingBackground: {};
    };
    /**
     * Check whether the queue has a waiting foreground loader with the given id.
     * If no `id` specified, it will check whether the queue has any waiting foreground loader.
     * @param id the optional id
     * @returns boolean
     */
    hasForeground(id?: string): boolean;
    /**
     * Check whether the queue has a waiting background loader with the given id.
     * If no `id` specified, it will check whether the queue has any waiting background loader.
     * @param id the optional id
     * @returns boolean
     */
    hasBackground(id?: string): boolean;
    /**
     * Start the foreground loading with a specified id.
     * The loading is only closed off when all IDs are called with stop() method.
     * @param id the optional id of the loading. id is set to 'default' by default.
     */
    start(id?: string): void;
    /**
     * Start the background loading with a specified id.
     * The loading is only closed off when all IDs are called with stopBackground() method.
     * @param id the optional id of the loading. id is set to 'default' by default.
     */
    startBackground(id?: string): void;
    /**
     * Stop a foreground loading with specific id
     * @param id the optional id to stop. If not provided, 'default' is used.
     * @returns Object
     */
    stop(id?: string): void;
    /**
     * Stop a background loading with specific id
     * @param id the optional id to stop. If not provided, 'default' is used.
     * @returns Object
     */
    stopBackground(id?: string): void;
    /**
     * Stop all the loadings including foreground and background
     */
    stopAll(): void;
    /**
     * Determine whether the loader is active
     * @returns true if the loader is active
     */
    private isActive;
    /**
     * Manage to close foreground loading
     */
    private foregroundCloseout;
    /**
     * Manage to close background loading
     */
    private backgroundCloseout;
}
